package com.psbc.cpsp.eden.common.utils;

import com.psbc.cpsp.eden.common.entity.Asset;
import com.psbc.cpsp.eden.common.vo.AssetImportVO;
import com.psbc.cpsp.eden.service.asset.IAssetClassificationService;
import com.psbc.cpsp.eden.service.asset.IAssetLocationService;
import com.psbc.cpsp.eden.enums.PurchaseTypeEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 资产导入监听器
 */
@Slf4j
public class AssetImportListener {

    private HttpServletRequest request;

    private final IAssetClassificationService assetClassificationService;
    private final IAssetLocationService assetLocationService;

    public AssetImportListener(IAssetClassificationService assetClassificationService,
            IAssetLocationService assetLocationService) {
        this.assetClassificationService = assetClassificationService;
        this.assetLocationService = assetLocationService;
        this.request = null;
    }

    public AssetImportListener(IAssetClassificationService assetClassificationService,
            IAssetLocationService assetLocationService,
            HttpServletRequest HTTPrequest) {
        this.assetClassificationService = assetClassificationService;
        this.assetLocationService = assetLocationService;
        this.request = HTTPrequest;
    }

    /**
     * 验证导入数据并转换为Asset实体
     *
     * @param importVOList 导入数据列表
     * @return 转换后的Asset对象列表，包含验证结果
     */
    public List<Asset> validateAndConvert(List<AssetImportVO> importVOList) {
        List<Asset> assetList = new ArrayList<>();

        for (AssetImportVO vo : importVOList) {
            Asset asset = new Asset();
            asset.setDelFlag("0");
            asset.setCreateTime(new Date());
            asset.setCreateUserId("admin"); // 使用默认的系统管理员

            StringBuilder errorMsg = new StringBuilder();

            // 验证和转换资产分类
            if (!StringUtils.hasText(vo.getClassificationName())) {
                errorMsg.append("资产分类不能为空; ");
            } else {
                try {
                    // 获取资产分类ID
                    Long classificationId = assetClassificationService
                            .selectIdByName(vo.getClassificationName());
                    if (classificationId == null) {
                        errorMsg.append("资产分类不存在; ");
                    } else {
                        asset.setClassificationId(classificationId);
                    }
                } catch (Exception e) {
                    errorMsg.append("查询资产分类失败; ");
                    log.error("查询资产分类异常", e);
                }
            }

            // 验证和转换资产名称
            if (!StringUtils.hasText(vo.getAssetName())) {
                errorMsg.append("资产名称不能为空; ");
            } else {
                asset.setAssetName(vo.getAssetName());
            }

            // 验证和转换品牌
            if (!StringUtils.hasText(vo.getBrand())) {
                errorMsg.append("品牌不能为空; ");
            } else {
                asset.setBrand(vo.getBrand());
            }

            // 转换型号（可选）
            asset.setModel(vo.getModel());

            // 转换设备序列号（可选）
            asset.setSerialNumber(vo.getSerialNumber());

            // 转换管理员
            asset.setAdminUserId(vo.getAdminUserName());

            // 验证和转换所属公司
            if (!StringUtils.hasText(vo.getCompanyName())) {
                errorMsg.append("所属公司不能为空; ");
            } else {
                // todo 公司名称转换为公司id
                asset.setCompanyId(null);
            }

            // 转换购置金额（可选）
            asset.setPurchaseAmount(vo.getPurchaseAmount());

            // 验证和转换所在位置
            if (!StringUtils.hasText(vo.getLocationName())) {
                errorMsg.append("所在位置不能为空; ");
            } else {
                try {
                    // 获取资产位置ID
                    Long locationId = assetLocationService.selectIdByName(vo.getLocationName());
                    if (locationId == null) {
                        errorMsg.append("所在位置不存在; ");
                    } else {
                        asset.setLocationId(locationId);
                    }
                } catch (Exception e) {
                    errorMsg.append("查询资产位置失败; ");
                    log.error("查询资产位置异常", e);
                }
            }

            // 转换购置时间（可选）
            asset.setPurchaseDate(vo.getPurchaseDate());

            // 验证和转换购置方式
            if (!StringUtils.hasText(vo.getPurchaseType())) {
                errorMsg.append("购置方式不能为空; ");
            } else {
                // 检查购置方式是否为枚举值，如果是文本名称则转换为代码
                String purchaseTypeCode = vo.getPurchaseType();
                if ("采购".equals(purchaseTypeCode)) {
                    purchaseTypeCode = PurchaseTypeEnum.BUY.getCode();
                } else if ("租赁".equals(purchaseTypeCode)) {
                    purchaseTypeCode = PurchaseTypeEnum.RENT.getCode();
                } else if (!PurchaseTypeEnum.isValid(purchaseTypeCode)) {
                    errorMsg.append("购置方式无效，应为采购或租赁; ");
                    purchaseTypeCode = null;
                }

                if (purchaseTypeCode != null) {
                    asset.setPurchaseType(purchaseTypeCode);
                }
            }

            // 转换入库时间（可选）
            asset.setStorageDate(vo.getStorageDate());

            // 转换预计使用期限（可选）
            asset.setExpectedUsefulLife(vo.getExpectedUsefulLife());

            // 转换备注（可选）
            asset.setRemark(vo.getRemark());

            // 转换保养到期时间（可选）
            asset.setMaintenanceDueDate(vo.getMaintenanceDueDate());

            // 转换保养说明（可选）
            asset.setMaintenanceDescription(vo.getMaintenanceDescription());

            // 转换预计折旧期限（可选）
            asset.setExpectedDepreciationPeriod(vo.getExpectedDepreciationPeriod());

            // 设置默认资产状态为空闲
            asset.setAssetStatus("空闲");

            // 记录导入结果
            vo.setImportResult(errorMsg.length() > 0 ? errorMsg.toString() : "导入成功");

            // 只有验证通过的才加入列表
            if (errorMsg.length() == 0) {
                assetList.add(asset);
            }
        }

        return assetList;
    }

}