package com.psbc.cpsp.eden.common.utils;

import com.psbc.cpsp.eden.common.entity.Asset;
import com.psbc.cpsp.eden.common.vo.AssetExportVO;
import com.psbc.cpsp.eden.common.vo.AssetImportVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import com.psbc.cpsp.eden.enums.PurchaseTypeEnum;

/**
 * Excel处理工具类
 */
@Slf4j
public class ExcelUtils {

    private static final String EXCEL_CONTENT_TYPE = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
    private static final String DATE_FORMAT = "yyyy-MM-dd";
    private static final SimpleDateFormat SDF = new SimpleDateFormat(DATE_FORMAT);

    /**
     * 导入资产Excel
     *
     * @param file Excel文件
     * @return 资产导入VO列表
     */
    public static List<AssetImportVO> importAssetExcel(MultipartFile file) {
        List<AssetImportVO> list = new ArrayList<>();

        try (InputStream is = file.getInputStream();
                Workbook workbook = WorkbookFactory.create(is)) {

            Sheet sheet = workbook.getSheetAt(0);

            // 跳过表头(前3行)，第一条记录从第4行开始
            int startRow = 3;
            int lastRowNum = sheet.getLastRowNum();

            for (int i = startRow; i <= lastRowNum; i++) {
                Row row = sheet.getRow(i);
                if (row == null) {
                    continue;
                }

                // 跳过空行
                if (isRowEmpty(row)) {
                    continue;
                }

                AssetImportVO vo = new AssetImportVO();

                // 根据资产导入模板顺序读取单元格数据
                vo.setClassificationName(getCellStringValue(row.getCell(0)));
                vo.setAssetName(getCellStringValue(row.getCell(1)));
                vo.setBrand(getCellStringValue(row.getCell(2)));
                vo.setModel(getCellStringValue(row.getCell(3)));
                vo.setSerialNumber(getCellStringValue(row.getCell(4)));
                vo.setAdminUserName(getCellStringValue(row.getCell(5)));
                vo.setCompanyName(getCellStringValue(row.getCell(6)));
                vo.setPurchaseAmount(getCellNumericValueAsBigDecimal(row.getCell(7)));
                vo.setLocationName(getCellStringValue(row.getCell(8)));
                vo.setPurchaseDate(getCellDateValue(row.getCell(9)));
                vo.setPurchaseType(getCellStringValue(row.getCell(10)));
                vo.setStorageDate(getCellDateValue(row.getCell(11)));
                vo.setExpectedUsefulLife(getCellNumericValueAsInteger(row.getCell(12)));
                vo.setRemark(getCellStringValue(row.getCell(13)));
                vo.setMaintenanceDueDate(getCellDateValue(row.getCell(14)));
                vo.setMaintenanceDescription(getCellStringValue(row.getCell(15)));
                vo.setExpectedDepreciationPeriod(getCellNumericValueAsInteger(row.getCell(16)));
                // 第17列是导入结果说明，不属于AssetImportVO对象，忽略

                list.add(vo);
            }

        } catch (Exception e) {
            log.error("导入Excel异常", e);
        }

        return list;
    }

    /**
     * 使用AssetImportListener导入Excel
     *
     * @param file     Excel文件
     * @param clazz    目标类
     * @param listener 导入监听器
     * @return 导入数据列表
     */
    public static <T> List<T> importExcel(MultipartFile file, Class<T> clazz, AssetImportListener listener) {
        if (file == null || clazz == null) {
            return new ArrayList<>();
        }

        if (clazz == AssetImportVO.class) {
            List<AssetImportVO> importList = importAssetExcel(file);
            // 这里实际不需要使用listener参数，listener主要用于其它处理场景
            return (List<T>) importList;
        }

        return new ArrayList<>();
    }

    /**
     * 将导入VO转换为实体对象
     *
     * @param importList 导入VO列表
     * @return 实体对象列表
     */
    public static List<Asset> convertImportData(List<AssetImportVO> importList) {
        if (importList == null || importList.isEmpty()) {
            return new ArrayList<>();
        }

        List<Asset> assetList = new ArrayList<>();
        for (AssetImportVO vo : importList) {
            Asset asset = new Asset();

            // 设置基本信息
            asset.setAssetName(vo.getAssetName());
            asset.setBrand(vo.getBrand());
            asset.setModel(vo.getModel());
            asset.setSerialNumber(vo.getSerialNumber());
            asset.setAdminUserId(vo.getAdminUserName());
            // todo 公司名称转换为公司id
            asset.setCompanyId(null);
            asset.setPurchaseAmount(vo.getPurchaseAmount());
            asset.setPurchaseDate(vo.getPurchaseDate());
            asset.setPurchaseType(vo.getPurchaseType());
            asset.setStorageDate(vo.getStorageDate());
            asset.setExpectedUsefulLife(vo.getExpectedUsefulLife());
            asset.setRemark(vo.getRemark());
            asset.setMaintenanceDueDate(vo.getMaintenanceDueDate());
            asset.setMaintenanceDescription(vo.getMaintenanceDescription());
            asset.setExpectedDepreciationPeriod(vo.getExpectedDepreciationPeriod());

            // 设置默认状态
            asset.setAssetStatus("空闲");
            asset.setDelFlag("0");
            asset.setCreateTime(new Date());

            assetList.add(asset);
        }

        return assetList;
    }

    /**
     * 导出资产Excel
     *
     * @param response HTTP响应
     * @param dataList 资产导出数据列表
     * @param fileName 导出文件名
     */
    public static void exportAssetExcel(HttpServletResponse response, List<AssetExportVO> dataList, String fileName) {
        try (Workbook workbook = new XSSFWorkbook()) {
            Sheet sheet = workbook.createSheet("资产清单");

            // 创建表头样式
            CellStyle headerStyle = workbook.createCellStyle();
            headerStyle.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
            headerStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            headerStyle.setBorderBottom(BorderStyle.THIN);
            headerStyle.setBorderLeft(BorderStyle.THIN);
            headerStyle.setBorderRight(BorderStyle.THIN);
            headerStyle.setBorderTop(BorderStyle.THIN);

            Font headerFont = workbook.createFont();
            headerFont.setBold(true);
            headerStyle.setFont(headerFont);

            // 创建表头
            Row headerRow = sheet.createRow(0);

            String[] headers = {
                    "资产编码", "资产分类", "资产名称", "资产状态", "品牌", "型号", "设备序列号",
                    "管理员", "所在位置", "购置时间", "购置方式", "购置金额(含税)", "入库时间",
                    "预计使用期限(月)", "备注", "资产照片链接", "保养到期时间", "保养说明", "预计折旧期限(月)"
            };

            for (int i = 0; i < headers.length; i++) {
                Cell cell = headerRow.createCell(i);
                cell.setCellValue(headers[i]);
                cell.setCellStyle(headerStyle);
                // 根据内容设置列宽
                sheet.setColumnWidth(i, 256 * 20);
            }

            // 创建数据行
            CellStyle dataStyle = workbook.createCellStyle();
            dataStyle.setBorderBottom(BorderStyle.THIN);
            dataStyle.setBorderLeft(BorderStyle.THIN);
            dataStyle.setBorderRight(BorderStyle.THIN);
            dataStyle.setBorderTop(BorderStyle.THIN);

            // 日期样式
            CellStyle dateStyle = workbook.createCellStyle();
            dateStyle.cloneStyleFrom(dataStyle);
            CreationHelper creationHelper = workbook.getCreationHelper();
            dateStyle.setDataFormat(creationHelper.createDataFormat().getFormat(DATE_FORMAT));

            int rowNum = 1;
            for (AssetExportVO vo : dataList) {
                Row row = sheet.createRow(rowNum++);

                // 资产编码
                createCell(row, 0, vo.getAssetCode(), dataStyle);
                // 资产分类
                createCell(row, 1, vo.getClassificationName(), dataStyle);
                // 资产名称
                createCell(row, 2, vo.getAssetName(), dataStyle);
                // 资产状态
                createCell(row, 3, vo.getAssetStatus(), dataStyle);
                // 品牌
                createCell(row, 4, vo.getBrand(), dataStyle);
                // 型号
                createCell(row, 5, vo.getModel(), dataStyle);
                // 设备序列号
                createCell(row, 6, vo.getSerialNumber(), dataStyle);
                // 管理员
                createCell(row, 7, vo.getAdminUserName(), dataStyle);
                // 所在位置
                createCell(row, 8, vo.getLocationName(), dataStyle);
                // 购置时间
                createDateCell(row, 9, vo.getPurchaseDate(), dateStyle);
                // 购置方式 - 转换编码为名称
                String purchaseTypeName = PurchaseTypeEnum.getNameByCode(vo.getPurchaseType());
                createCell(row, 10, purchaseTypeName, dataStyle);
                // 购置金额(含税)
                createCell(row, 11, vo.getPurchaseAmount(), dataStyle);
                // 入库时间
                createDateCell(row, 12, vo.getStorageDate(), dateStyle);
                // 预计使用期限(月)
                createCell(row, 13, vo.getExpectedUsefulLife(), dataStyle);
                // 备注
                createCell(row, 14, vo.getRemark(), dataStyle);
                // 资产照片链接
                createCell(row, 15, vo.getAssetPhoto(), dataStyle);
                // 保养到期时间
                createDateCell(row, 16, vo.getMaintenanceDueDate(), dateStyle);
                // 保养说明
                createCell(row, 17, vo.getMaintenanceDescription(), dataStyle);
                // 预计折旧期限(月)
                createCell(row, 18, vo.getExpectedDepreciationPeriod(), dataStyle);
            }

            // 设置响应头
            response.setContentType(EXCEL_CONTENT_TYPE);
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename="
                    + URLEncoder.encode(fileName + ".xlsx", "UTF-8"));

            // 写入响应
            workbook.write(response.getOutputStream());

        } catch (Exception e) {
            log.error("导出Excel异常", e);
        }
    }

    /**
     * 通用导出Excel方法
     *
     * @param list      数据列表
     * @param sheetName sheet名称
     * @param clazz     对象类型
     * @param fileName  文件名
     * @param response  HTTP响应
     */
    public static <T> void exportExcel(List<T> list, String sheetName, Class<T> clazz, String fileName,
            HttpServletResponse response) {
        if (list == null || list.isEmpty() || clazz == null) {
            return;
        }

        if (clazz == AssetExportVO.class) {
            exportAssetExcel(response, (List<AssetExportVO>) list, fileName);
            return;
        }

        // 默认导出处理
        try (Workbook workbook = new XSSFWorkbook()) {
            // 创建sheet
            Sheet sheet = workbook.createSheet(sheetName);

            // 设置响应头
            response.setContentType(EXCEL_CONTENT_TYPE);
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename="
                    + URLEncoder.encode(fileName + ".xlsx", "UTF-8"));

            // 写入响应
            workbook.write(response.getOutputStream());
        } catch (Exception e) {
            log.error("导出Excel异常", e);
        }
    }

    /**
     * 通用导出Excel方法 - 不带文件名版本
     */
    public static <T> void exportExcel(List<T> list, String sheetName, Class<T> clazz, HttpServletResponse response) {
        String fileName = sheetName + "_" + new SimpleDateFormat("yyyyMMdd").format(new Date());
        exportExcel(list, sheetName, clazz, fileName, response);
    }

    /**
     * 创建Excel导入模板
     */
    public static List<AssetImportVO> createExcelTemplate() {
        // 创建空的模板数据列表
        List<AssetImportVO> templateList = new ArrayList<>();

        // 添加一条示例数据
        AssetImportVO example = new AssetImportVO();
        example.setClassificationName("办公设备");
        example.setAssetName("办公电脑");
        example.setBrand("联想");
        example.setModel("ThinkPad X1");
        example.setSerialNumber("SN12345678");
        example.setAdminUserName("张三");
        example.setCompanyName("总公司");
        example.setPurchaseAmount(new BigDecimal("5999.00"));
        example.setLocationName("总部办公室");
        example.setPurchaseType("采购");
        example.setRemark("这是一条示例数据");

        templateList.add(example);

        return templateList;
    }

    /**
     * 获取单元格字符串值
     */
    private static String getCellStringValue(Cell cell) {
        if (cell == null) {
            return "";
        }

        String value = "";
        switch (cell.getCellType()) {
            case STRING:
                value = cell.getStringCellValue();
                break;
            case NUMERIC:
                if (DateUtil.isCellDateFormatted(cell)) {
                    value = SDF.format(cell.getDateCellValue());
                } else {
                    value = String.valueOf((long) cell.getNumericCellValue());
                }
                break;
            case BOOLEAN:
                value = String.valueOf(cell.getBooleanCellValue());
                break;
            case FORMULA:
                try {
                    value = cell.getStringCellValue();
                } catch (Exception e) {
                    value = String.valueOf(cell.getNumericCellValue());
                }
                break;
            default:
                break;
        }
        return value;
    }

    /**
     * 获取单元格日期值
     */
    private static Date getCellDateValue(Cell cell) {
        if (cell == null) {
            return null;
        }

        Date date = null;
        switch (cell.getCellType()) {
            case NUMERIC:
                if (DateUtil.isCellDateFormatted(cell)) {
                    date = cell.getDateCellValue();
                }
                break;
            case STRING:
                String dateStr = cell.getStringCellValue();
                if (dateStr != null && !dateStr.isEmpty()) {
                    try {
                        date = SDF.parse(dateStr);
                    } catch (ParseException e) {
                        log.error("日期格式解析异常: {}", dateStr, e);
                    }
                }
                break;
            default:
                break;
        }
        return date;
    }

    /**
     * 获取单元格数值为BigDecimal
     */
    private static BigDecimal getCellNumericValueAsBigDecimal(Cell cell) {
        if (cell == null) {
            return null;
        }

        BigDecimal value = null;
        switch (cell.getCellType()) {
            case NUMERIC:
                value = new BigDecimal(cell.getNumericCellValue());
                break;
            case STRING:
                String numStr = cell.getStringCellValue();
                if (numStr != null && !numStr.isEmpty()) {
                    try {
                        value = new BigDecimal(numStr);
                    } catch (NumberFormatException e) {
                        log.error("数值格式解析异常: {}", numStr, e);
                    }
                }
                break;
            default:
                break;
        }
        return value;
    }

    /**
     * 获取单元格数值为Integer
     */
    private static Integer getCellNumericValueAsInteger(Cell cell) {
        if (cell == null) {
            return null;
        }

        Integer value = null;
        switch (cell.getCellType()) {
            case NUMERIC:
                value = (int) cell.getNumericCellValue();
                break;
            case STRING:
                String numStr = cell.getStringCellValue();
                if (numStr != null && !numStr.isEmpty()) {
                    try {
                        value = Integer.valueOf(numStr);
                    } catch (NumberFormatException e) {
                        log.error("整数格式解析异常: {}", numStr, e);
                    }
                }
                break;
            default:
                break;
        }
        return value;
    }

    /**
     * 创建单元格并设置值
     */
    private static void createCell(Row row, int columnIndex, Object value, CellStyle style) {
        Cell cell = row.createCell(columnIndex);
        if (value instanceof String) {
            cell.setCellValue((String) value);
        } else if (value instanceof Integer) {
            cell.setCellValue((Integer) value);
        } else if (value instanceof Long) {
            cell.setCellValue((Long) value);
        } else if (value instanceof Double) {
            cell.setCellValue((Double) value);
        } else if (value instanceof BigDecimal) {
            cell.setCellValue(((BigDecimal) value).doubleValue());
        } else if (value instanceof Boolean) {
            cell.setCellValue((Boolean) value);
        } else if (value != null) {
            cell.setCellValue(value.toString());
        }
        cell.setCellStyle(style);
    }

    /**
     * 创建日期单元格
     */
    private static void createDateCell(Row row, int columnIndex, Date date, CellStyle style) {
        Cell cell = row.createCell(columnIndex);
        if (date != null) {
            cell.setCellValue(date);
        }
        cell.setCellStyle(style);
    }

    /**
     * 判断行是否为空
     */
    private static boolean isRowEmpty(Row row) {
        if (row == null) {
            return true;
        }

        for (int i = row.getFirstCellNum(); i < row.getLastCellNum(); i++) {
            Cell cell = row.getCell(i);
            if (cell != null && cell.getCellType() != CellType.BLANK) {
                return false;
            }
        }
        return true;
    }

    /**
     * 创建导入模板 (旧方法)
     */
    public static void createImportTemplate(HttpServletResponse response) throws IOException {
        try (Workbook workbook = new XSSFWorkbook()) {
            Sheet sheet = workbook.createSheet("资产导入模板");

            // 设置列宽
            for (int i = 0; i < 18; i++) {
                sheet.setColumnWidth(i, 256 * 20);
            }

            // 创建标题行样式
            CellStyle titleStyle = workbook.createCellStyle();
            titleStyle.setAlignment(HorizontalAlignment.CENTER);
            Font titleFont = workbook.createFont();
            titleFont.setBold(true);
            titleFont.setFontHeightInPoints((short) 14);
            titleStyle.setFont(titleFont);

            // 创建标题行
            Row titleRow = sheet.createRow(0);
            Cell titleCell = titleRow.createCell(0);
            titleCell.setCellValue("资产导入模板");
            titleCell.setCellStyle(titleStyle);
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 17));

            // 创建说明行样式
            CellStyle descriptionStyle = workbook.createCellStyle();
            descriptionStyle.setAlignment(HorizontalAlignment.LEFT);
            Font descriptionFont = workbook.createFont();
            descriptionFont.setFontHeightInPoints((short) 11);
            descriptionStyle.setFont(descriptionFont);

            // 创建说明行
            Row descriptionRow = sheet.createRow(1);
            Cell descriptionCell = descriptionRow.createCell(0);
            descriptionCell.setCellValue("导入说明: 带*号的为必填项，请按照模板格式填写，日期格式为yyyy-MM-dd。");
            descriptionCell.setCellStyle(descriptionStyle);
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 17));

            // 创建表头样式
            CellStyle headerStyle = workbook.createCellStyle();
            headerStyle.setAlignment(HorizontalAlignment.CENTER);
            headerStyle.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
            headerStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            headerStyle.setBorderBottom(BorderStyle.THIN);
            headerStyle.setBorderLeft(BorderStyle.THIN);
            headerStyle.setBorderRight(BorderStyle.THIN);
            headerStyle.setBorderTop(BorderStyle.THIN);
            Font headerFont = workbook.createFont();
            headerFont.setBold(true);
            headerStyle.setFont(headerFont);

            // 创建表头
            Row headerRow = sheet.createRow(2);
            String[] headers = {
                    "*资产分类", "*资产名称", "*品牌", "型号", "设备序列号",
                    "管理员", "*所属公司", "购置金额(含税)", "*所在位置",
                    "购置时间", "*购置方式", "入库时间", "预计使用期限(月)",
                    "备注", "保养到期日期", "保养说明", "预计折旧期限(月)", "导入结果说明"
            };

            for (int i = 0; i < headers.length; i++) {
                Cell cell = headerRow.createCell(i);
                cell.setCellValue(headers[i]);
                cell.setCellStyle(headerStyle);
            }

            // 设置数据验证 - 购置方式下拉列表
            DataValidationHelper dvHelper = sheet.getDataValidationHelper();

            // 购置方式列(下标10)，有效值：采购、租赁
            CellRangeAddressList purchaseTypeRange = new CellRangeAddressList(3, 1000, 10, 10);
            DataValidationConstraint purchaseTypeConstraint = dvHelper.createExplicitListConstraint(
                    new String[] { "采购", "租赁" });
            DataValidation purchaseTypeValidation = dvHelper.createValidation(purchaseTypeConstraint,
                    purchaseTypeRange);
            purchaseTypeValidation.setShowErrorBox(true);
            sheet.addValidationData(purchaseTypeValidation);

            // 设置响应头
            response.setContentType(EXCEL_CONTENT_TYPE);
            response.setCharacterEncoding("UTF-8");
            String fileName = URLEncoder.encode("资产导入模板.xlsx", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName);

            // 写入响应
            workbook.write(response.getOutputStream());
        }
    }
}