package com.psbc.cpsp.eden.controller.asset;

import com.psbc.cpsp.entity.CpspReponse;
import com.psbc.cpsp.eden.common.entity.AssetClassification;
import com.psbc.cpsp.eden.enums.StatusEnum;
import com.psbc.cpsp.eden.common.vo.TreeNode;
import com.psbc.cpsp.eden.service.asset.IAssetClassificationService;
import com.github.pagehelper.PageHelper;
import com.psbc.cpsp.entity.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 资产分类Controller
 *
 * @author eden
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/asset/classifications")
public class AssetClassificationController {
    @Autowired
    private IAssetClassificationService assetClassificationService;

    /**
     * 查询资产分类列表
     */
    @GetMapping("/list")
    public CpspReponse<PageInfo<AssetClassification>> list(AssetClassification classification,
            @RequestParam(defaultValue = "1") int pageNum,
            @RequestParam(defaultValue = "10") int pageSize) {
        log.info("查询资产分类列表, 参数: {}, pageNum: {}, pageSize: {}", classification, pageNum, pageSize);
        PageHelper.startPage(pageNum, pageSize);
        List<AssetClassification> list = assetClassificationService.selectList(classification);

        // 使用GitHub的PageInfo获取分页信息
        com.github.pagehelper.PageInfo<AssetClassification> githubPageInfo = new com.github.pagehelper.PageInfo<>(list);

        // 转换为自定义的PageInfo
        PageInfo<AssetClassification> pageInfo = new PageInfo<>(
                githubPageInfo.getPageSize(),
                githubPageInfo.getPageNum(),
                githubPageInfo.getTotal(),
                githubPageInfo.getList());

        return CpspReponse.ok(pageInfo);
    }

    /**
     * 查询资产分类树结构
     */
    @GetMapping("/tree")
    public CpspReponse<List<TreeNode>> tree(AssetClassification classification) {
        log.info("查询资产分类树结构, 参数: {}", classification);
        try {
            List<TreeNode> tree = assetClassificationService.selectTree(classification);
            log.info("查询资产分类树结构成功, 返回节点数: {}", tree != null ? tree.size() : 0);

            // 检查树结构中的根节点是否有children属性
            if (tree != null && !tree.isEmpty()) {
                for (TreeNode node : tree) {
                    log.debug("根节点: {} - {} 子节点数: {}",
                            node.getId(), node.getLabel(),
                            node.getChildren() != null ? node.getChildren().size() : 0);
                }
            }

            return CpspReponse.ok(tree);
        } catch (Exception e) {
            log.error("查询资产分类树结构出错", e);
            return CpspReponse.error("999999", "查询资产分类树结构出错: " + e.getMessage());
        }
    }

    /**
     * 根据父ID查询子节点列表
     */
    @GetMapping("/children/{parentId}")
    public CpspReponse<PageInfo<AssetClassification>> children(@NotNull @PathVariable Long parentId,
            @RequestParam(defaultValue = "1") int pageNum,
            @RequestParam(defaultValue = "10") int pageSize) {
        log.info("查询子节点列表, parentId: {},  pageNum: {}, pageSize: {}",
                parentId, pageNum, pageSize);
        PageHelper.startPage(pageNum, pageSize);
        List<AssetClassification> list = assetClassificationService.selectChildrenByParentId(parentId);

        // 使用GitHub的PageInfo获取分页信息
        com.github.pagehelper.PageInfo<AssetClassification> githubPageInfo = new com.github.pagehelper.PageInfo<>(list);

        // 转换为自定义的PageInfo
        PageInfo<AssetClassification> pageInfo = new PageInfo<>(
                githubPageInfo.getPageSize(),
                githubPageInfo.getPageNum(),
                githubPageInfo.getTotal(),
                githubPageInfo.getList());

        return CpspReponse.ok(pageInfo);
    }

    /**
     * 获取资产分类详细信息
     */
    @GetMapping("/{id}")
    public CpspReponse<AssetClassification> getInfo(@NotNull @PathVariable Long id) {
        log.info("获取资产分类详情, id: {}", id);
        AssetClassification classification = assetClassificationService.selectById(id);
        if (classification == null) {
            log.warn("获取资产分类详情失败，未找到资产分类, id: {}", id);
            return CpspReponse.error("999999", "未找到资产分类");
        }
        return CpspReponse.ok(classification);
    }

    /**
     * 新增资产分类
     */
    @PostMapping("/create")
    public CpspReponse<AssetClassification> create(@Validated @RequestBody AssetClassification classification) {
        log.info("新增资产分类, 参数: {}", classification);

        if (classification.getCreateUserId() == null || classification.getCreateUserId().isEmpty()) {
            classification.setCreateUserId("system");
        }

        // 校验状态值是否有效
        if (!StatusEnum.isValid(classification.getStatus())) {
            return CpspReponse.error("999999", "无效的状态值");
        }
        // 校验编码是否唯一
        if (assetClassificationService.checkCodeUnique(classification)) {
            return CpspReponse.error("999999", "资产分类编码已存在");
        }
        if (assetClassificationService.checkClassificationNameExists(classification)) {
            return CpspReponse.error("999999", "资产分类名称已存在");
        }
        // 设置创建时间
        classification.setCreateTime(new Date());
        return CpspReponse.ok(assetClassificationService.insert(classification));
    }

    /**
     * 修改资产分类
     */
    @PostMapping("/update/{id}")
    public CpspReponse<AssetClassification> update(@NotNull @PathVariable Long id,
            @Validated @RequestBody AssetClassification classification) {
        log.info("修改资产分类, id: {}, 参数: {}", id, classification);

        classification.setId(id);

        // 设置必要的默认值
        if (classification.getLastModUserId() == null || classification.getLastModUserId().isEmpty()) {
            classification.setLastModUserId("system");
        }

        // 校验状态值是否有效
        if (classification.getStatus() != null && !StatusEnum.isValid(classification.getStatus())) {
            return CpspReponse.error("999999", "无效的状态值");
        }
        // 校验编码是否唯一
        if (assetClassificationService.checkCodeUnique(classification)) {
            return CpspReponse.error("999999", "资产分类编码已存在");
        }
        if (assetClassificationService.checkClassificationNameExists(classification)) {
            return CpspReponse.error("999999", "资产分类名称已存在");
        }
        // 设置修改时间
        classification.setLastModTime(new Date());
        return CpspReponse.ok(assetClassificationService.update(classification));
    }

    /**
     * 禁用资产分类
     */
    @PostMapping("/{id}/status/disable")
    public CpspReponse<Void> disable(@NotNull @PathVariable Long id) {
        log.info("禁用资产分类, id: {}", id);
        try {
            // 使用专用的状态更新方法
            String lastModUserId ="system";
            boolean success = assetClassificationService.updateStatus(id, StatusEnum.DISABLED.getCode(), lastModUserId);

            if (success) {
                return CpspReponse.ok();
            } else {
                return CpspReponse.error("999999", "禁用资产分类失败");
            }
        } catch (Exception e) {
            log.error("禁用资产分类异常", e);
            return CpspReponse.error("999999", "禁用资产分类失败");
        }
    }

    /**
     * 启用资产分类
     */
    @PostMapping("/{id}/status/enable")
    public CpspReponse<Void> enable(@NotNull @PathVariable Long id) {
        log.info("启用资产分类, id: {}, lastModUserId: {}", id);
        try {
            // 使用专用的状态更新方法
            String lastModUserId ="system";
            boolean success = assetClassificationService.updateStatus(id, StatusEnum.ENABLED.getCode(), lastModUserId);

            if (success) {
                return CpspReponse.ok();
            } else {
                return CpspReponse.error("999999", "启用资产分类失败");
            }
        } catch (Exception e) {
            log.error("启用资产分类异常", e);
            return CpspReponse.error("999999", "启用资产分类失败");
        }
    }

    /**
     * 删除资产分类
     */
    @PostMapping("/delete/{id}")
    public CpspReponse<Void> delete(@NotNull @PathVariable Long id) {
        log.info("删除资产分类, id: {}", id);

        try {
            // 直接执行删除，无需先查询
            String lastModUserId ="system";
            assetClassificationService.deleteById(id, lastModUserId);
            return CpspReponse.ok();
        } catch (Exception e) {
            log.error("删除资产分类异常", e);
            return CpspReponse.error("999999", "删除资产分类失败");
        }
    }

    /**
     * 批量删除资产分类
     */
    @PostMapping("/deleteBatch")
    public CpspReponse<Void> deleteBatch(@NotEmpty @RequestBody List<Long> ids) {
        log.info("批量删除资产分类, ids: {}", ids);
        String lastModUserId = "system";
        assetClassificationService.deleteBatchIds(ids, lastModUserId);
        return CpspReponse.ok();
    }
}