package com.psbc.cpsp.eden.controller.asset;

import com.github.pagehelper.PageHelper;
import com.psbc.cpsp.entity.PageInfo;
import com.psbc.cpsp.eden.common.entity.Asset;
import com.psbc.cpsp.eden.common.entity.AssetLabel;
import com.psbc.cpsp.entity.CpspReponse;
import com.psbc.cpsp.eden.common.utils.AssetImportListener;
import com.psbc.cpsp.eden.common.utils.ExcelUtils;
import com.psbc.cpsp.eden.common.vo.AssetExportVO;
import com.psbc.cpsp.eden.common.vo.AssetImportVO;
import com.psbc.cpsp.eden.service.asset.IAssetClassificationService;
import com.psbc.cpsp.eden.service.asset.IAssetLabelService;
import com.psbc.cpsp.eden.service.asset.IAssetLocationService;
import com.psbc.cpsp.eden.service.asset.IAssetService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 资产Controller
 *
 * @author eden
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/asset")
public class AssetController {
    @Autowired
    @Qualifier("basicAssetService")
    private IAssetService assetService;

    @Autowired
    private IAssetClassificationService assetClassificationService;

    @Autowired
    private IAssetLocationService assetLocationService;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private IAssetLabelService assetLabelService;

    /**
     * 查询资产列表
     */
    @GetMapping("/list")
    public CpspReponse<PageInfo<Asset>> list(Asset asset,
                                             @RequestParam(defaultValue = "1") int pageNum,
                                             @RequestParam(defaultValue = "10") int pageSize,
                                             @RequestParam(required = false) String adminUserId) {
        log.info("查询资产列表, 参数: {}, pageNum: {}, pageSize: {}", asset, pageNum, pageSize);
        // 设置管理员ID查询条件
        if (adminUserId != null && !adminUserId.isEmpty()) {
            asset.setAdminUserId(adminUserId);
            log.info("设置管理员ID查询条件: {}", adminUserId);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<Asset> list = assetService.selectList(asset);

        // 使用GitHub的PageInfo获取分页信息
        com.github.pagehelper.PageInfo<Asset> githubPageInfo = new com.github.pagehelper.PageInfo<>(list);

        // 转换为自定义的PageInfo
        PageInfo<Asset> pageInfo = new PageInfo<>(
                githubPageInfo.getPageSize(),
                githubPageInfo.getPageNum(),
                githubPageInfo.getTotal(),
                githubPageInfo.getList());

        return CpspReponse.ok(pageInfo);
    }

    /**
     * 获取资产详细信息
     */
    @GetMapping("/{assetCode}")
    public CpspReponse<Asset> getInfo(@NotBlank @PathVariable String assetCode) {
        log.info("获取资产详细信息, assetCode: {}", assetCode);

        // 根据资产编码查询资产
        Asset asset = assetService.selectByCode(assetCode);
        if (asset == null) {
            return CpspReponse.error("999999", "未找到资产");
        }

        return CpspReponse.ok(asset);
    }

    /**
     * 新增资产
     */
    @PostMapping("/create")
    public CpspReponse<Asset> create(@Validated @RequestBody Asset asset) {
        log.info("新增资产, 参数: {}", asset);

        // 设置默认值
        if (asset.getCreateUserId() == null || asset.getCreateUserId().isEmpty()) {
            asset.setCreateUserId("1");
        }

        // 校验编码是否唯一
        log.info("开始校验资产编码唯一性: {}", asset.getAssetCode());
        boolean isUnique = assetService.checkCodeUnique(asset);
        if (!isUnique) {
            log.warn("资产编码已存在: {}", asset.getAssetCode());
            return CpspReponse.error("999999", "资产编码已存在");
        }
        log.info("资产编码唯一性校验通过");

        // 设置创建时间
        asset.setCreateTime(new Date());

        try {
            return CpspReponse.ok(assetService.insert(asset));
        } catch (RuntimeException e) {
            log.error("新增资产失败", e);
            return CpspReponse.error("999999", e.getMessage());
        }
    }

    /**
     * 修改资产
     */
    @PostMapping("/update/{assetCode}")
    public CpspReponse<Asset> update(@NotBlank @PathVariable String assetCode,
                                     @Validated @RequestBody Asset asset) {
        log.info("修改资产, assetCode: {}, 参数: {}", assetCode, asset);

        // 根据资产编码获取原资产信息
        Asset existingAsset = assetService.selectByCode(assetCode);
        if (existingAsset == null) {
            return CpspReponse.error("999999", "未找到资产");
        }

        // 设置资产ID
        asset.setId(existingAsset.getId());
        // 设置资产编码
        asset.setAssetCode(assetCode);

        // 设置必要的默认值
        if (asset.getLastModUserId() == null || asset.getLastModUserId().isEmpty()) {
            asset.setLastModUserId("1");
        }

        // 设置修改时间
        asset.setLastModTime(new Date());
        return CpspReponse.ok(assetService.update(asset));
    }

    /**
     * 更新资产状态
     */
    @PostMapping("/{id}/status/{status}")
    public CpspReponse<Void> updateStatus(@NotNull @PathVariable Long id,
                                          @NotBlank @PathVariable String status) {
        log.info("更新资产状态, id: {}, status: {}", id, status);
        try {
            // 使用专用的状态更新方法
            String lastModUserId = "system";
            boolean success = assetService.updateStatus(id, status, lastModUserId);

            if (success) {
                return CpspReponse.ok();
            } else {
                return CpspReponse.error("999999", "更新资产状态失败");
            }
        } catch (Exception e) {
            log.error("更新资产状态异常", e);
            return CpspReponse.error("999999", "更新资产状态失败");
        }
    }

    /**
     * 删除资产
     */
    @PostMapping("/delete/{assetCode}")
    public CpspReponse<Void> delete(@NotBlank @PathVariable String assetCode) {
        log.info("删除资产, assetCode: {}", assetCode);

        // 根据资产编码查询资产
        Asset asset = assetService.selectByCode(assetCode);
        if (asset == null) {
            return CpspReponse.error("999999", "未找到资产");
        }

        // 删除资产
        assetService.deleteById(asset.getId(), getCurrentUserId());
        return CpspReponse.ok();
    }

    /**
     * 批量删除资产
     */
    @PostMapping("/batch/delete")
    public CpspReponse<Void> deleteBatch(@NotEmpty @RequestBody List<Long> ids) {
        log.info("批量删除资产, ids: {}", ids);
        assetService.deleteBatchIds(ids, getCurrentUserId());
        return CpspReponse.ok();
    }

    /**
     * 校验资产编码唯一性
     */
    @GetMapping("/check/code")
    public CpspReponse<Boolean> checkCode(Asset asset) {
        log.info("校验资产编码唯一性, asset: {}", asset);
        return CpspReponse.ok(assetService.checkCodeUnique(asset));
    }

    /**
     * 导入资产数据
     */
    @PostMapping("/import")
    public CpspReponse<String> importAsset(@RequestPart("file") MultipartFile file,
                                           @RequestParam(defaultValue = "false") boolean updateSupport) {
        log.info("导入资产数据, 文件名: {}, 大小: {}, 是否更新已存在数据: {}",
                file.getOriginalFilename(), file.getSize(), updateSupport);

        try {
            // 使用EasyExcel导入数据
            List<AssetImportVO> importList = ExcelUtils.importExcel(file, AssetImportVO.class,
                    new AssetImportListener(assetClassificationService, assetLocationService, request));
            log.info("导入数据解析完成，共 {} 条记录", importList.size());

            // 将导入VO转换为实体并保存
            String operName = getUsername();
            String message = assetService.importAsset(ExcelUtils.convertImportData(importList), updateSupport,
                    operName);
            return CpspReponse.ok(message);
        } catch (Exception e) {
            log.error("导入资产数据异常", e);
            return CpspReponse.error("999999", "导入资产数据失败：" + e.getMessage());
        }
    }

    /**
     * 导出资产数据
     */
    @GetMapping("/export")
    public void exportAsset(Asset asset, HttpServletResponse response,
            @RequestParam(required = false) String maintenanceExpired,
            @RequestParam(required = false) String adminUserId) {
        log.info("导出资产数据, 参数: {}, maintenanceExpired: {}", asset, maintenanceExpired);

        // 设置维护到期筛选条件
        if (maintenanceExpired != null && !maintenanceExpired.isEmpty()) {
            asset.setMaintenanceExpired(maintenanceExpired);
            log.info("设置维护到期筛选条件: {}", maintenanceExpired);
        }

            // 设置管理员ID查询条件
            if (adminUserId != null && !adminUserId.isEmpty()) {
                asset.setAdminUserId(adminUserId);
                log.info("设置管理员ID查询条件: {}", adminUserId);
            }

        // 查询导出数据
            List<AssetExportVO> list = assetService.exportAsset(asset);
        log.info("查询到符合条件的资产数据 {} 条", list.size());

            // 设置导出文件名
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            String fileName = "资产清单" + sdf.format(new Date());

        // 获取每个资产的公司名称和管理员名称
        for (AssetExportVO vo : list) {
            Asset assetTemp = new Asset();
            assetTemp.setCompanyId(vo.getCompanyId());
            String companyName = assetService.getAssetCompanyName(assetTemp);
            vo.setCompanyName(companyName);

            assetTemp.setAdminUserId(vo.getAdminUserId());
            String adminName = assetService.getAssetAdminName(assetTemp);
            vo.setAdminUserName(adminName);
        }

        // 导出Excel
        ExcelUtils.exportExcel(list, "资产清单", AssetExportVO.class, fileName, response);
    }

    /**
     * 获取用户名
     */
    private String getUsername() {
        // 从请求头或Session中获取用户信息
        String username = request.getHeader("X-Username");
        if (username == null || username.isEmpty()) {
            username = "admin"; // 默认用户名
        }
        return username;
    }


    /**
     * 获取当前用户ID
     */
    private String getCurrentUserId() {
        // 返回固定的用户ID "admin"
        log.debug("使用固定用户ID: admin");
        return "admin";
    }

    /**
     * 获取标签信息
     */
    @GetMapping("/label")
    public CpspReponse<AssetLabel> getLabel() {
        log.info("获取标签信息");
        AssetLabel label = assetLabelService.selectByAll();

        if (label == null) {
            // 如果未找到标签，创建默认标签
            label = new AssetLabel();
            label.setTemplateType("1");
            label.setLabelItems("1,2,3");
            label.setPaperType("1");
            label.setCreateUserId(getCurrentUserId());
            label.setCreateTime(new Date());
            label.setDelFlag("0");
            assetLabelService.insert(label);
            log.info("创建默认标签: {}", label);
        }

        return CpspReponse.ok(label);
    }

    /**
     * 下载资产导入模板
     */
    @GetMapping("/template/download")
    public void downloadTemplate(HttpServletResponse response) {
        log.info("下载资产导入模板");
        try {
            // 设置响应头
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            String fileName = "资产导入模板.xlsx";
            fileName = java.net.URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName);

            // 填充模板数据
            List<AssetImportVO> list = ExcelUtils.createExcelTemplate();

            // 导出模板
            ExcelUtils.exportExcel(list, "资产数据", AssetImportVO.class, response);
        } catch (IOException e) {
            log.error("下载模板失败", e);
        }
    }
}