package com.psbc.cpsp.eden.controller.asset;

import com.psbc.cpsp.entity.CpspReponse;
import com.psbc.cpsp.eden.common.entity.AssetLocation;
import com.psbc.cpsp.eden.enums.StatusEnum;
import com.psbc.cpsp.eden.common.vo.TreeNode;
import com.psbc.cpsp.eden.service.asset.IAssetLocationService;
import com.github.pagehelper.PageHelper;
import com.psbc.cpsp.entity.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 资产位置Controller
 *
 * @author eden
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/asset/locations")
public class AssetLocationController {
    @Autowired
    private IAssetLocationService assetLocationService;

    /**
     * 查询资产位置列表
     */
    @GetMapping("/list")
    public CpspReponse<PageInfo<AssetLocation>> list(AssetLocation location,
            @RequestParam(defaultValue = "1") int pageNum,
            @RequestParam(defaultValue = "10") int pageSize) {
        log.info("查询资产位置列表, 参数: {}, pageNum: {}, pageSize: {}", location, pageNum, pageSize);
        PageHelper.startPage(pageNum, pageSize);
        List<AssetLocation> list = assetLocationService.selectList(location);

        // 使用GitHub的PageInfo获取分页信息
        com.github.pagehelper.PageInfo<AssetLocation> githubPageInfo = new com.github.pagehelper.PageInfo<>(list);

        // 转换为自定义的PageInfo
        PageInfo<AssetLocation> pageInfo = new PageInfo<>(
                githubPageInfo.getPageSize(),
                githubPageInfo.getPageNum(),
                githubPageInfo.getTotal(),
                githubPageInfo.getList());

        return CpspReponse.ok(pageInfo);
    }

    /**
     * 查询资产位置树结构
     */
    @GetMapping("/tree")
    public CpspReponse<List<TreeNode>> tree(AssetLocation location) {
        log.info("查询资产位置树结构, 参数: {}", location);
        return CpspReponse.ok(assetLocationService.selectTree(location));
    }

    /**
     * 根据父ID查询子节点列表
     */
    @GetMapping("/children/{parentId}")
    public CpspReponse<PageInfo<AssetLocation>> children(@NotNull @PathVariable Long parentId,
            @RequestParam(defaultValue = "1") int pageNum,
            @RequestParam(defaultValue = "10") int pageSize) {
        log.info("查询子节点列表, parentId: {}, pageNum: {}, pageSize: {}",
                parentId, pageNum, pageSize);
        PageHelper.startPage(pageNum, pageSize);
        List<AssetLocation> list = assetLocationService.selectChildrenByParentId(parentId);

        // 使用GitHub的PageInfo获取分页信息
        com.github.pagehelper.PageInfo<AssetLocation> githubPageInfo = new com.github.pagehelper.PageInfo<>(list);

        // 转换为自定义的PageInfo
        PageInfo<AssetLocation> pageInfo = new PageInfo<>(
                githubPageInfo.getPageSize(),
                githubPageInfo.getPageNum(),
                githubPageInfo.getTotal(),
                githubPageInfo.getList());

        return CpspReponse.ok(pageInfo);
    }

    /**
     * 获取资产位置详细信息
     */
    @GetMapping("/{id}")
    public CpspReponse<AssetLocation> getInfo(@NotNull @PathVariable Long id) {
        log.info("获取资产位置详情, id: {}", id);
        AssetLocation location = assetLocationService.selectById(id);
        if (location == null) {
            log.warn("获取资产位置详情失败，未找到资产位置, id: {}", id);
            return CpspReponse.error("999999", "未找到资产位置");
        }
        return CpspReponse.ok(location);
    }

    /**
     * 新增资产位置
     */
    @PostMapping("/create")
    public CpspReponse<AssetLocation> create(@Validated @RequestBody AssetLocation location) {
        log.info("新增资产位置, 参数: {}", location);
        // 设置必要的默认值
        if (location.getStatus() == null || location.getStatus().isEmpty()) {
            location.setStatus(StatusEnum.ENABLED.getCode());
        }
        if (location.getCreateUserId() == null || location.getCreateUserId().isEmpty()) {
            location.setCreateUserId("system");
        }

        // 校验状态值是否有效
        if (!StatusEnum.isValid(location.getStatus())) {
            return CpspReponse.error("999999", "无效的状态值");
        }
        // 校验编码是否唯一
        if (assetLocationService.checkCodeUnique(location)) {
            return CpspReponse.error("999999", "资产位置编码已存在");
        }
        // 校验名称是否唯一
        if (assetLocationService.checkNameUnique(location)) {
            return CpspReponse.error("999999", "资产位置名称已存在");
        }
        // 设置创建时间
        location.setCreateTime(new Date());
        return CpspReponse.ok(assetLocationService.insert(location));
    }

    /**
     * 修改资产位置
     */
    @PostMapping("/update/{id}")
    public CpspReponse<AssetLocation> update(@NotNull @PathVariable Long id,
            @Validated @RequestBody AssetLocation location) {
        log.info("修改资产位置, id: {}, 参数: {}", id, location);

        location.setId(Long.valueOf(id));

        // 设置必要的默认值
        if (location.getLastModUserId() == null || location.getLastModUserId().isEmpty()) {
            location.setLastModUserId("system");
        }

        // 校验状态值是否有效
        if (location.getStatus() != null && !StatusEnum.isValid(location.getStatus())) {
            return CpspReponse.error("999999", "无效的状态值");
        }
        // 校验编码是否唯一
        if (assetLocationService.checkCodeUnique(location)) {
            return CpspReponse.error("999999", "资产位置编码已存在");
        }
        // 校验名称是否唯一
        if (assetLocationService.checkNameUnique(location)) {
            return CpspReponse.error("999999", "资产位置名称已存在");
        }
        // 设置修改时间
        location.setLastModTime(new Date());
        return CpspReponse.ok(assetLocationService.update(location));
    }

    /**
     * 禁用资产位置
     */
    @PostMapping("/{id}/status/disable")
    public CpspReponse<Void> disable(@NotNull @PathVariable Long id) {
        log.info("禁用资产位置, id: {}", id);
        try {
            // 使用专用的状态更新方法
            String lastModUserId = "system";
            boolean success = assetLocationService.updateStatus(id, StatusEnum.DISABLED.getCode(), lastModUserId);

            if (success) {
                return CpspReponse.ok();
            } else {
                return CpspReponse.error("999999", "禁用资产位置失败");
            }
        } catch (Exception e) {
            log.error("禁用资产位置异常", e);
            return CpspReponse.error("999999", "禁用资产位置失败");
        }
    }

    /**
     * 启用资产位置
     */
    @PostMapping("/{id}/status/enable")
    public CpspReponse<Void> enable(@NotNull @PathVariable Long id) {
        log.info("启用资产位置, id: {}", id);
        try {
            // 使用专用的状态更新方法
            String lastModUserId = "system";
            boolean success = assetLocationService.updateStatus(id, StatusEnum.ENABLED.getCode(), lastModUserId);

            if (success) {
                return CpspReponse.ok();
            } else {
                return CpspReponse.error("999999", "启用资产位置失败");
            }
        } catch (Exception e) {
            log.error("启用资产位置异常", e);
            return CpspReponse.error("999999", "启用资产位置失败");
        }
    }

    /**
     * 删除资产位置
     */
    @PostMapping("/delete/{id}")
    public CpspReponse<Void> delete(@NotNull @PathVariable Long id) {
        log.info("删除资产位置, id: {}", id);
        String lastModUserId =  "system";
        assetLocationService.deleteById(id, lastModUserId);
        return CpspReponse.ok();
    }

    /**
     * 批量删除资产位置
     */
    @PostMapping("/batch/delete")
    public CpspReponse<Void> deleteBatch(@NotEmpty @RequestBody List<Long> ids) {
        log.info("批量删除资产位置, ids: {}", ids);
        String lastModUserId = "system";
        assetLocationService.deleteBatchIds(ids, lastModUserId);
        return CpspReponse.ok();
    }

    /**
     * 检查编码唯一性
     */
    @GetMapping("/check/code")
    public CpspReponse<Boolean> checkCode(AssetLocation location) {
        log.info("检查编码唯一性, 参数: {}", location);
        return CpspReponse.ok(assetLocationService.checkCodeUnique(location));
    }
}