package com.psbc.cpsp.eden.service.asset.impl;

import com.psbc.cpsp.eden.common.entity.AssetClassification;
import com.psbc.cpsp.eden.enums.DelFlagEnum;
import com.psbc.cpsp.eden.common.vo.TreeNode;
import com.psbc.cpsp.eden.mapper.asset.AssetClassificationMapper;
import com.psbc.cpsp.eden.service.asset.IAssetClassificationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * 资产分类Service实现类
 */
@Slf4j
@Service
public class AssetClassificationServiceImpl implements IAssetClassificationService {


    private final AssetClassificationMapper assetClassificationMapper;

    public AssetClassificationServiceImpl(AssetClassificationMapper assetClassificationMapper) {
        this.assetClassificationMapper = assetClassificationMapper;
    }

    @Override
    public List<AssetClassification> selectList(AssetClassification classification) {
        return assetClassificationMapper.selectList(classification);
    }

    @Override
    public List<TreeNode> selectTree(AssetClassification classification) {
        if (classification == null) {
            classification = new AssetClassification();
        }

        // 获取所有节点
        List<TreeNode> allNodes = assetClassificationMapper.selectTree(classification);
        if (allNodes == null || allNodes.isEmpty()) {
            return allNodes;
        }

        // 构建树结构
        List<TreeNode> rootNodes = buildTree(allNodes);
        return rootNodes;
    }

    /**
     * 构建树结构
     *
     * @param allNodes 所有节点
     * @return 树根节点列表
     */
    private List<TreeNode> buildTree(List<TreeNode> allNodes) {
        // 使用Map存储所有节点，以便快速查找
        Map<Long, TreeNode> nodeMap = new HashMap<>();
        List<TreeNode> rootNodes = new ArrayList<>();

        log.info("开始构建树结构，总节点数: {}", allNodes.size());

        // 第一次遍历，将所有节点放入Map
        for (TreeNode node : allNodes) {
            nodeMap.put(node.getId(), node);
            log.debug("节点ID: {}, 父节点ID: {}, 标签: {}", node.getId(), node.getParentId(), node.getLabel());
        }

        // 第二次遍历，构建树结构
        for (TreeNode node : allNodes) {
            Long parentId = node.getParentId();
            if (parentId == null) {
                // 如果没有父节点，则为根节点
                log.debug("添加根节点: {}, 标签: {}", node.getId(), node.getLabel());
                rootNodes.add(node);
            } else {
                // 如果有父节点，将当前节点添加到父节点的children列表中
                TreeNode parentNode = nodeMap.get(parentId);
                if (parentNode != null) {
                    if (parentNode.getChildren() == null) {
                        parentNode.setChildren(new ArrayList<>());
                    }
                    log.debug("添加子节点: {}, 标签: {} 到父节点: {}", node.getId(), node.getLabel(), parentId);
                    parentNode.getChildren().add(node);
                } else {
                    // 如果找不到父节点，作为根节点处理
                    log.warn("找不到父节点ID为: {} 的节点，将节点ID: {}, 标签: {} 作为根节点处理", parentId, node.getId(), node.getLabel());
                    rootNodes.add(node);
                }
            }
        }

        log.info("树结构构建完成，根节点数: {}", rootNodes.size());
        return rootNodes;
    }

    @Override
    public List<AssetClassification> selectChildrenByParentId(Long parentId) {
        return assetClassificationMapper.selectChildrenByParentId(parentId);
    }

    @Override
    public AssetClassification selectById(Long id) {
        log.info("根据ID查询资产分类, id: {}", id);
        try {
            AssetClassification result = assetClassificationMapper.selectById(id);
            if (result == null) {
                log.warn("未找到对应的资产分类, id: {}", id);
            }
            return result;
        } catch (Exception e) {
            log.error("查询资产分类异常", e);
            return null;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AssetClassification insert(AssetClassification classification) {
        if (classification.getCreateTime() == null) {
            classification.setCreateTime(new Date());
        }
        if (classification.getDelFlag() == null) {
            classification.setDelFlag(DelFlagEnum.NORMAL.getCode());
        }
        int rows = assetClassificationMapper.insert(classification);
        if (rows > 0) {
            log.info("新增资产分类成功, id: {}", classification.getId());
        } else {
            log.error("新增资产分类失败");
        }
        return classification;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AssetClassification update(AssetClassification classification) {
        classification.setLastModTime(new Date());
        int rows = assetClassificationMapper.update(classification);
        if (rows > 0) {
            log.info("更新资产分类成功, id: {}", classification.getId());
        } else {
            log.error("更新资产分类失败, id: {}", classification.getId());
        }
        return classification;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteById(Long id, String lastModUserId) {
        try {
            // todo 校验是否被使用，注意
            int rows = assetClassificationMapper.deleteById(id, lastModUserId);
            if (rows > 0) {
                log.info("删除资产分类成功, id: {}", id);
            } else {
                log.error("删除资产分类失败, id: {}", id);
            }
        } catch (Exception e) {
            log.error("删除资产分类异常, id: {}", id, e);
            throw e;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBatchIds(List<Long> ids, String lastModUserId) {
        if (ids == null || ids.isEmpty()) {
            return;
        }

        try {
            log.info("批量删除资产分类，数量: {}", ids.size());

            int rows = assetClassificationMapper.deleteBatchIds(ids, lastModUserId);
            if (rows > 0) {
                log.info("批量删除资产分类成功, 删除数量: {}", rows);
            } else {
                log.error("批量删除资产分类失败");
            }
        } catch (Exception e) {
            log.error("批量删除资产分类异常", e);
            throw e;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateStatus(Long id, String status, String lastModUserId) {
        log.info("更新资产分类状态, id: {}, status: {}, lastModUserId: {}", id, status, lastModUserId);
        try {

            Date lastModTime = new Date();
            int rows = assetClassificationMapper.updateStatus(id, status, lastModUserId, lastModTime);
            if (rows > 0) {
                log.info("更新资产分类状态成功");
                return true;
            } else {
                log.error("更新资产分类状态失败");
                return false;
            }
        } catch (Exception e) {
            log.error("更新资产分类状态异常", e);
            return false;
        }
    }

    @Override
    public boolean checkCodeUnique(AssetClassification classification) {

        int count = assetClassificationMapper.checkCodeUnique(
                classification.getClassificationCode(),
                classification.getId());
        return count > 0;
    }

    /**
     * 根据分类名称查询分类ID
     *
     * @param classificationName 分类名称
     * @return 分类ID
     */
    @Override
    public Long selectIdByName(String classificationName) {
        return assetClassificationMapper.selectIdByName(classificationName);
    }

    /**
     * 检查分类名称是否存在
     *
     * @return 是否存在
     */
    @Override
    public boolean checkClassificationNameExists(AssetClassification classification) {
        return assetClassificationMapper.checkNameExists(classification.getClassificationName(),classification.getId()) > 0;
    }
}