package com.psbc.cpsp.eden.service.asset.impl;

import com.psbc.cpsp.eden.common.entity.AssetLabel;
import com.psbc.cpsp.eden.enums.LabelTemplateTypeEnum;
import com.psbc.cpsp.eden.enums.LabelItemEnum;
import com.psbc.cpsp.eden.enums.PaperTypeEnum;
import com.psbc.cpsp.eden.mapper.asset.AssetLabelMapper;
import com.psbc.cpsp.eden.service.asset.IAssetLabelService;
import com.psbc.cpsp.entity.CpspReponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.List;

import com.psbc.cpsp.eden.common.response.AssetResult;

/**
 * 资产标签Service实现类
 */
@Service
public class AssetLabelServiceImpl implements IAssetLabelService {

    @Autowired
    private AssetLabelMapper assetLabelMapper;

    /**
     * 查询资产标签列表
     *
     * @param label 资产标签信息
     * @return 资产标签集合
     */
    @Override
    public List<AssetLabel> selectList(AssetLabel label) {
        return assetLabelMapper.selectList(label);
    }

    /**
     * 根据ID查询资产标签
     *
     * @param id 资产标签ID
     * @return 资产标签信息
     */
    @Override
    public AssetLabel selectById(Long id) {
        return assetLabelMapper.selectById(id);
    }

    /**
     * 新增资产标签
     *
     * @param label 资产标签信息
     * @return 资产标签信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public AssetLabel insert(AssetLabel label) {
        AssetResult<AssetLabel> validationAssetResult = validateAssetLabel(label);
        if (!validationAssetResult.isSuccess()) {
            throw new IllegalArgumentException(validationAssetResult.getMessage());
        }
        // 设置默认值
        if (label.getDelFlag() == null || label.getDelFlag().isEmpty()) {
            label.setDelFlag("0");
        }

        // 执行插入操作
        assetLabelMapper.insert(label);
        return label;
    }

    /**
     * 修改资产标签
     *
     * @param label 资产标签信息
     * @return 资产标签信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public AssetLabel update(AssetLabel label) {
        AssetResult<AssetLabel> validationAssetResult = validateAssetLabel(label);
        if (!validationAssetResult.isSuccess()) {
            throw new IllegalArgumentException(validationAssetResult.getMessage());
        }
        // 执行更新操作
        assetLabelMapper.update(label);
        return assetLabelMapper.selectById(label.getId());
    }

    /**
     * 删除资产标签
     *
     * @param id            资产标签ID
     * @param lastModUserId 最后修改人ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteById(Long id, String lastModUserId) {
        assetLabelMapper.deleteById(id, lastModUserId);
    }

    /**
     * 批量删除资产标签
     *
     * @param ids           需要删除的数据ID集合
     * @param lastModUserId 最后修改人ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBatchIds(List<Long> ids, String lastModUserId) {
        assetLabelMapper.deleteBatchIds(ids, lastModUserId);
    }

    /**
     * 验证资产标签数据
     */
    @Override
    public AssetResult<AssetLabel> validateAssetLabel(AssetLabel label) {
        // 验证模板类型
        if (!Arrays.asList("1", "2", "3", "4").contains(label.getTemplateType())) {
            return AssetResult.error("模板类型无效");
        }

        // 验证标签集合
        if (StringUtils.isEmpty(label.getLabelItems())) {
            return AssetResult.error("标签集合不能为空");
        }

        // 验证标签数量
        String[] items = label.getLabelItems().split(",");
        int maxLabels = Integer.parseInt(label.getTemplateType());
        if (items.length > maxLabels) {
            return AssetResult.error("选择的标签数量超过模板限制");
        }

        // 验证标签项
        for (String item : items) {
            if (!Arrays.asList("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11").contains(item)) {
                return AssetResult.error("标签项无效");
            }
        }

        // 验证纸张类型
        /*
         * if (!Arrays.asList("1", "2", "3").contains(label.getPaperType())) {
         * return Result.error("纸张类型无效");
         * }
         * 
         * // 验证标签尺寸
         * if ("1".equals(label.getPaperType())) {
         * if (StringUtils.isEmpty(label.getLabelWidth()) ||
         * StringUtils.isEmpty(label.getLabelHeight())) {
         * return Result.error("标签专用纸必须指定宽度和高度");
         * }
         * try {
         * int width = Integer.parseInt(label.getLabelWidth());
         * int height = Integer.parseInt(label.getLabelHeight());
         * if (width < 20 || width > 100 || height < 20 || height > 100) {
         * return Result.error("标签尺寸必须在20-100mm之间");
         * }
         * } catch (NumberFormatException e) {
         * return Result.error("标签尺寸必须是有效的数字");
         * }
         * }
         */

        return AssetResult.success(label);
    }

    private boolean isValidTemplateType(String templateType) {
        LabelTemplateTypeEnum template = LabelTemplateTypeEnum.getByCode(templateType);
        return template != null;
    }

    private boolean isValidLabelItem(String item) {
        LabelItemEnum labelItem = LabelItemEnum.getByCode(item);
        return labelItem != null;
    }

    private boolean isValidPaperType(String paperType) {
        PaperTypeEnum paper = PaperTypeEnum.getByCode(paperType);
        return paper != null;
    }

    @Override
    public AssetLabel selectByAll() {
        AssetLabel label = new AssetLabel();
        List<AssetLabel> list = assetLabelMapper.selectList(label);
        return list.isEmpty() ? null : list.get(0);
    }
}