package com.psbc.cpsp.eden.service.asset.impl;

import com.psbc.cpsp.eden.common.entity.AssetLocation;
import com.psbc.cpsp.eden.enums.DelFlagEnum;
import com.psbc.cpsp.eden.common.vo.TreeNode;
import com.psbc.cpsp.eden.mapper.asset.AssetLocationMapper;
import com.psbc.cpsp.eden.service.asset.IAssetLocationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

/**
 * 资产位置Service实现类
 */
@Slf4j
@Service
public class AssetLocationServiceImpl implements IAssetLocationService {


    @Autowired
    private AssetLocationMapper assetLocationMapper;

    @Override
    public List<AssetLocation> selectList(AssetLocation location) {
        // 设置租户ID
        if (location == null) {
            location = new AssetLocation();
        }

        return assetLocationMapper.selectList(location);
    }

    @Override
    public List<TreeNode> selectTree(AssetLocation location) {
        // 设置租户ID
        if (location == null) {
            location = new AssetLocation();
        }

        // 获取所有节点
        List<TreeNode> allNodes = assetLocationMapper.selectTree(location);
        if (allNodes == null || allNodes.isEmpty()) {
            return allNodes;
        }

        // 构建树结构
        List<TreeNode> rootNodes = buildTree(allNodes);
        return rootNodes;
    }

    /**
     * 构建树结构
     *
     * @param allNodes 所有节点
     * @return 树根节点列表
     */
    private List<TreeNode> buildTree(List<TreeNode> allNodes) {
        // 使用Map存储所有节点，以便快速查找
        Map<Long, TreeNode> nodeMap = new HashMap<>();
        List<TreeNode> rootNodes = new ArrayList<>();

        // 第一次遍历，将所有节点放入Map
        for (TreeNode node : allNodes) {
            nodeMap.put(node.getId(), node);
        }

        // 第二次遍历，构建树结构
        for (TreeNode node : allNodes) {
            Long parentId = node.getParentId();
            if (parentId == null) {
                // 如果没有父节点，则为根节点
                rootNodes.add(node);
            } else {
                // 如果有父节点，将当前节点添加到父节点的children列表中
                TreeNode parentNode = nodeMap.get(parentId);
                if (parentNode != null) {
                    if (parentNode.getChildren() == null) {
                        parentNode.setChildren(new ArrayList<>());
                    }
                    parentNode.getChildren().add(node);
                } else {
                    // 如果找不到父节点，作为根节点处理
                    rootNodes.add(node);
                }
            }
        }

        return rootNodes;
    }

    @Override
    public List<AssetLocation> selectChildrenByParentId(Long parentId) {
        return assetLocationMapper.selectChildrenByParentId(parentId);
    }

    @Override
    public AssetLocation selectById(Long id) {
        log.info("根据ID查询资产位置, id: {}", id);
        try {
            AssetLocation result = assetLocationMapper.selectById(id);
            if (result == null) {
                log.warn("未找到对应的资产位置, id: {}", id);
            }
            return result;
        } catch (Exception e) {
            log.error("查询资产位置异常", e);
            return null;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AssetLocation insert(AssetLocation location) {
        if (location.getCreateTime() == null) {
            location.setCreateTime(new Date());
        }
        if (location.getDelFlag() == null) {
            location.setDelFlag(DelFlagEnum.NORMAL.getCode());
        }

        try {
            int rows = assetLocationMapper.insert(location);
            if (rows > 0) {
                log.info("新增资产位置成功, id: {}", location.getId());
            } else {
                log.error("新增资产位置失败");
            }
        } catch (Exception e) {
            log.error("新增资产位置异常", e);
            throw e;
        }

        return location;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AssetLocation update(AssetLocation location) {
        location.setLastModTime(new Date());

        try {
            int rows = assetLocationMapper.update(location);
            if (rows > 0) {
                log.info("更新资产位置成功, id: {}", location.getId());
            } else {
                log.error("更新资产位置失败, id: {}", location.getId());
            }
        } catch (Exception e) {
            log.error("更新资产位置异常", e);
            throw e;
        }

        return location;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteById(Long id, String lastModUserId) {
        try {
            // todo
            int rows = assetLocationMapper.deleteById(id, lastModUserId);
            if (rows > 0) {
                log.info("删除资产位置成功, id: {}", id);
            } else {
                log.error("删除资产位置失败, id: {}", id);
            }
        } catch (Exception e) {
            log.error("删除资产位置异常, id: {}", id, e);
            throw e;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBatchIds(List<Long> ids, String lastModUserId) {
        if (ids == null || ids.isEmpty()) {
            return;
        }

        try {
            log.info("批量删除资产位置，数量: {}", ids.size());
            // todo
            int rows = assetLocationMapper.deleteBatchIds(ids, lastModUserId);
            if (rows > 0) {
                log.info("批量删除资产位置成功, 删除数量: {}", rows);
            } else {
                log.error("批量删除资产位置失败");
            }
        } catch (Exception e) {
            log.error("批量删除资产位置异常", e);
            throw e;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateStatus(Long id, String status, String lastModUserId) {
        log.info("更新资产位置状态, id: {}, status: {}, lastModUserId: {}", id, status, lastModUserId);
        try {

            Date lastModTime = new Date();
            int rows = assetLocationMapper.updateStatus(id, status, lastModUserId, lastModTime);
            if (rows > 0) {
                log.info("更新资产位置状态成功, id: {}", id);
                return true;
            } else {
                log.error("更新资产位置状态失败, id: {}", id);
                return false;
            }
        } catch (Exception e) {
            log.error("更新资产位置状态异常, id: {}", id, e);
            throw e;
        }
    }

    @Override
    public boolean checkCodeUnique(AssetLocation location) {
        int count = assetLocationMapper.checkCodeUnique(
                location.getLocationCode(),
                location.getId());
        return count > 0;
    }


    @Override
    public boolean checkNameUnique(AssetLocation location) {

        int count = assetLocationMapper.checkNameUnique(
                location.getLocationName(),
                location.getId());
        return count > 0;
    }

    /**
     * 根据位置名称查询位置ID
     *
     * @param locationName 位置名称
     * @return 位置ID
     */
    @Override
    public Long selectIdByName(String locationName) {
        return assetLocationMapper.selectIdByName(locationName);
    }
}