package com.psbc.cpsp.eden.service.asset.impl;

import com.psbc.cpsp.eden.common.entity.Asset;
import com.psbc.cpsp.eden.common.entity.AssetClassification;
import com.psbc.cpsp.eden.common.entity.AssetLocation;
import com.psbc.cpsp.eden.common.utils.AdminUserCache;
import com.psbc.cpsp.eden.common.utils.CompanyCache;
import com.psbc.cpsp.eden.common.vo.AssetExportVO;
import com.psbc.cpsp.eden.mapper.asset.AssetClassificationMapper;
import com.psbc.cpsp.eden.mapper.asset.AssetLocationMapper;
import com.psbc.cpsp.eden.mapper.asset.AssetMapper;
import com.psbc.cpsp.eden.service.asset.IAssetService;
import com.psbc.cpsp.entity.BizException;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 资产Service实现类
 */
@Slf4j
@Service("basicAssetService")
public class AssetServiceImpl implements IAssetService {

    @Autowired
    private AssetMapper assetMapper;

    @Autowired
    private AssetClassificationMapper assetClassificationMapper;

    @Autowired
    private AssetLocationMapper assetLocationMapper;

    /**
     * 查询资产列表
     *
     * @param asset 资产信息
     * @return 资产集合
     */
    @Override
    public List<Asset> selectList(Asset asset) {
        return assetMapper.selectList(asset);
    }

    /**
     * 根据ID查询资产
     *
     * @param id 资产ID
     * @return 资产信息
     */
    @Override
    public Asset selectById(Long id) {
        return assetMapper.selectById(id);
    }

    /**
     * 新增资产
     *
     * @param asset 资产信息
     * @return 资产信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Asset insert(Asset asset) {
        // 设置默认值
        if (asset.getDelFlag() == null || asset.getDelFlag().isEmpty()) {
            asset.setDelFlag("0");
        }


        // 再次检查资产编码是否已存在（防止并发问题）
        if (asset.getAssetCode() != null && !asset.getAssetCode().isEmpty()) {
            log.info("Service层再次校验资产编码唯一性 - 编码: {}",
                    asset.getAssetCode());

            // 直接使用SQL查询检查编码是否存在
            Asset existingAsset = assetMapper.selectByCode(asset.getAssetCode());
            if (existingAsset != null) {
                log.error("资产编码 [{}] 已存在于数据库中！", asset.getAssetCode());
                throw new BizException("400", "资产编码 [" + asset.getAssetCode() + "] 已存在，请更换一个唯一的编码");
            }

            int count = assetMapper.checkCodeUnique(asset.getAssetCode(), null);
            log.info("Service层再次校验资产编码唯一性 - 结果: {}", count == 0 ? "唯一" : "已存在");
            if (count > 0) {
                log.error("资产编码 [{}] 已存在！", asset.getAssetCode());
                throw new BizException("400", "资产编码 [" + asset.getAssetCode() + "] 已存在，请更换一个唯一的编码");
            }
            // 新增默认空闲状态
            asset.setAssetStatus("空闲");
        }

        try {
            // 执行插入操作
            log.info("开始执行资产插入操作，资产编码: {}", asset.getAssetCode());
            int rows = assetMapper.insert(asset);
            log.info("资产插入执行结果: {} 行受影响", rows);
            return asset;
        } catch (Exception e) {
            log.error("资产插入操作异常: {}", e.getMessage(), e);
            throw new BizException("500", "资产创建失败: " + e.getMessage());
        }
    }

    /**
     * 修改资产
     *
     * @param asset 资产信息
     * @return 资产信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Asset update(Asset asset) {
        // 执行更新操作
        assetMapper.update(asset);
        return assetMapper.selectById(asset.getId());
    }

    /**
     * 更新资产状态
     *
     * @param id            资产ID
     * @param status        状态
     * @param lastModUserId 最后修改人ID
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateStatus(Long id, String status, String lastModUserId) {
        Date lastModTime = new Date();
        return assetMapper.updateStatus(id, status, lastModUserId, lastModTime) > 0;
    }

    /**
     * 删除资产
     *
     * @param id            资产ID
     * @param lastModUserId 最后修改人ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteById(Long id, String lastModUserId) {
        assetMapper.deleteById(id, lastModUserId);
    }

    /**
     * 批量删除资产
     *
     * @param ids           需要删除的数据ID集合
     * @param lastModUserId 最后修改人ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBatchIds(List<Long> ids, String lastModUserId) {
        assetMapper.deleteBatchIds(ids, lastModUserId);
    }

    /**
     * 校验资产编码是否唯一
     *
     * @param asset 资产信息
     * @return 结果
     */
    @Override
    public boolean checkCodeUnique(Asset asset) {
        log.info("校验资产编码唯一性 - 编码: {}, ID: {}",
                asset.getAssetCode(), asset.getId());
        int count = assetMapper.checkCodeUnique(asset.getAssetCode(),asset.getId());
        log.info("校验资产编码唯一性 - 结果: {}", count == 0 ? "唯一" : "已存在");
        return count == 0;
    }

    /**
     * 导入资产数据
     *
     * @param assetList     资产数据列表
     * @param updateSupport 是否更新支持，如果已存在，是否更新
     * @param operName      操作人
     * @return 结果消息
     */
    @Override
    @Transactional(rollbackFor = Exception.class, noRollbackFor = { BizException.class })
    public String importAsset(List<Asset> assetList, boolean updateSupport, String operName) {
        if (assetList == null || assetList.isEmpty()) {
            return "导入资产数据不能为空！";
        }

        log.info("开始导入资产数据，数量: {}, 是否支持更新: {}, 操作人: {}", assetList.size(), updateSupport, operName);

        // 结果计数器
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        for (Asset asset : assetList) {
            try {
                // 设置默认值
                if (asset.getCreateUserId() == null || asset.getCreateUserId().isEmpty()) {
                    asset.setCreateUserId(operName);
                }
                asset.setCreateTime(new Date());
                asset.setDelFlag("0");

                // 校验资产编码是否唯一
                Asset existAsset = null;
                if (asset.getAssetCode() != null && !asset.getAssetCode().isEmpty()) {
                    // 如果提供了资产编码，检查是否存在
                    int count = assetMapper.checkCodeUnique(asset.getAssetCode(), null);
                    if (count > 0) {
                        if (!updateSupport) {
                            // 如果不允许更新，则记为失败
                            failureNum++;
                            failureMsg.append("<br/>").append(failureNum).append("、资产 ").append(asset.getAssetName())
                                    .append(" 导入失败：资产编码 [").append(asset.getAssetCode()).append("] 已存在");
                            continue;
                        }
                        // 如果允许更新，则获取现有资产
                        existAsset = assetMapper.selectByCode(asset.getAssetCode());
                    }
                }

                if (existAsset != null && updateSupport) {
                    // 更新已有资产
                    asset.setId(existAsset.getId());
                    asset.setLastModUserId(operName);
                    asset.setLastModTime(new Date());
                    assetMapper.update(asset);
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、资产 ").append(asset.getAssetName())
                            .append(" 更新成功");
                } else if (existAsset == null) {
                    // 新增资产
                    // 如果未提供资产编码，则自动生成
                    if (asset.getAssetCode() == null || asset.getAssetCode().isEmpty()) {
                        asset.setAssetCode(generateAssetCode());
                    }

                    assetMapper.insert(asset);
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、资产 ").append(asset.getAssetName())
                            .append(" 导入成功");
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、资产 ").append(asset.getAssetName())
                            .append(" 已存在");
                }
            } catch (Exception e) {
                failureNum++;
                String message = "<br/>" + failureNum + "、资产 " + asset.getAssetName() + " 导入失败：";
                failureMsg.append(message).append(e.getMessage());
                log.error("资产导入失败: 资产名称={}, 资产编码={}, 异常信息={}",
                        asset.getAssetName(), asset.getAssetCode(), e.getMessage(), e);
            }
        }

        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            return failureMsg.toString();
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
            return successMsg.toString();
        }
    }

    /**
     * 导出资产数据
     *
     * @param asset 查询条件
     * @return 资产导出数据列表
     */
    @Override
    public List<AssetExportVO> exportAsset(Asset asset) {
        log.info("开始导出资产数据，查询条件: {}", asset);

        // 获取资产列表
        List<Asset> assetList = assetMapper.selectList(asset);
        log.info("查询到 {} 条资产记录", assetList.size());

        List<AssetExportVO> exportList = new ArrayList<>();

        // 遍历资产列表，转换为导出对象
        for (Asset item : assetList) {
            AssetExportVO exportVO = new AssetExportVO();
            BeanUtils.copyProperties(item, exportVO);

            // 获取资产分类信息
            if (item.getClassificationId() != null) {
                log.info("查询资产分类信息, classificationId: {}", item.getClassificationId());
                AssetClassification classification = assetClassificationMapper.selectById(item.getClassificationId());
                if (classification != null) {
                    exportVO.setClassificationName(classification.getClassificationName());
                    log.info("找到资产分类: {}", classification.getClassificationName());
                } else {
                    log.warn("未找到资产分类, classificationId: {}", item.getClassificationId());
                }
            }

            // 获取资产位置信息
            if (item.getLocationId() != null) {
                log.info("查询资产位置信息, locationId: {}", item.getLocationId());
                AssetLocation location = assetLocationMapper.selectById(item.getLocationId());
                if (location != null) {
                    exportVO.setLocationName(location.getLocationName());
                    log.info("找到资产位置: {}", location.getLocationName());
                } else {
                    log.warn("未找到资产位置, locationId: {}", item.getLocationId());
                }
            }

            // 获取管理员信息
            if (item.getAdminUserId() != null) {
                String adminUserName = AdminUserCache.getAdminUserName(item.getAdminUserId());
                exportVO.setAdminUserName(adminUserName);
            }

            // 获取公司信息
            if (item.getCompanyId() != null) {
                String companyName = CompanyCache.getCompanyName(item.getCompanyId());
                exportVO.setCompanyName(companyName);
            }

            exportList.add(exportVO);
        }

        log.info("资产数据导出转换完成，共 {} 条记录", exportList.size());
        return exportList;
    }

    /**
     * 生成资产编码
     * 规则：ZC + 年月日 + 4位序列号
     */
    private String generateAssetCode() {
        // 获取当前日期
        Date now = new Date();
        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("yyyyMMdd");
        String dateStr = sdf.format(now);

        // 获取当天最大序列号
        String prefix = "ZC" + dateStr;
        int maxSequence = assetMapper.getMaxSequenceByPrefix(prefix);

        // 生成新的序列号
        int newSequence = maxSequence + 1;
        String sequenceStr = String.format("%04d", newSequence);

        return prefix + sequenceStr;
    }

    @Override
    public Asset selectByCode(String assetCode) {
        log.info("根据资产编码查询资产, assetCode: {}", assetCode);
        Asset asset = new Asset();
        asset.setAssetCode(assetCode);
        asset.setDelFlag("0");
        List<Asset> list = assetMapper.selectList(asset);
        return list != null && !list.isEmpty() ? list.get(0) : null;
    }

    /**
     * 获取资产管理员名称
     *
     * @param asset 资产信息
     * @return 管理员名称
     */
    @Override
    public String getAssetAdminName(Asset asset) {
        if (asset == null || asset.getAdminUserId() == null) {
            return "未知管理员";
        }
        return AdminUserCache.getAdminUserName(asset.getAdminUserId());
    }

    /**
     * 获取资产所属公司名称
     *
     * @param asset 资产信息
     * @return 公司名称
     */
    @Override
    public String getAssetCompanyName(Asset asset) {
        if (asset == null || asset.getCompanyId() == null) {
            return "未知公司";
        }
        return CompanyCache.getCompanyName(asset.getCompanyId());
    }
}